/**
 * Developed by Mr. LiChi
 */

const millisecondsPerWeek = 1000 * 60 * 60 * 24 * 30;
const oneMonthAgo = (new Date()).getTime() - millisecondsPerWeek;

// Add click event for "Add" button
document.getElementById("addCustomeSite").onclick = function () {
    var newSite = document.getElementById("customeSite").value;
    var errorMsg = document.getElementById("errorMsg");
    console.log("等待保存新网址：" + newSite);

    if (newSite == "") {
        saveFailed(errorMsg, "不能为空！")
        return;
    } else if (!newSite.toLowerCase().startsWith("https") && !newSite.toLowerCase().startsWith("http") && !newSite.toLowerCase().startsWith("ftp")) {
        saveFailed(errorMsg, "必须以“https”或“http”或“ftp”开头！")
        return;
    }

    chrome.storage.sync.get(['filterSites'], function (result) {
        let currentSites = result["filterSites"];
        console.log('自定义网址有：' + currentSites);

        if (currentSites == undefined) {
            currentSites = [];
            currentSites.push(newSite);
            chrome.storage.sync.set({
                "filterSites": currentSites
            }, function () {
                saveSuccessful(errorMsg, currentSites);
                listAllSitesOnPage();
                deleteHistoryOfCustomSite(newSite, () => {
                    document.getElementById("promptMsg").innerHTML = "清理完成！";
                    setTimeout(() => {
                        document.getElementById("promptMsg").style.display = "none";
                    }, 1000);
                });
            });
        } else if (currentSites != undefined) {
            if (currentSites.length >= 5) {
                saveFailed(errorMsg, "最多只允许添加5个自定义网址！");
                return;
            }

            currentSites.push(newSite);
            chrome.storage.sync.set({
                "filterSites": currentSites
            }, function () {
                saveSuccessful(errorMsg, currentSites);
                listAllSitesOnPage();
                deleteHistoryOfCustomSite(newSite, () => {
                    document.getElementById("promptMsg").innerHTML = "清理完成！";
                    setTimeout(() => {
                        document.getElementById("promptMsg").style.display = "none";
                    }, 1000);
                });
            });
        }
    });
}

/**
 * Delete history of custom site.
 *  by default, up to 5000 matching records within a month will be deleted.
 *  Call the function when user add custom site.
 */
function deleteHistoryOfCustomSite(site, callback) {
    chrome.history.search({ text: site, startTime: oneMonthAgo, maxResults: 5000 }, results => {
        console.log("Clearing old history of " + site + "...");

        // document.getElementById("promptMsg").innerHTML = "正在清理中(一个月内至多5000条匹配)...";
        // document.getElementById("promptMsg").style.display = "block";

        results.forEach(({ url }) => {
            if (url.startsWith(site)) {
                console.log("Deleting history: " + url);
                setTimeout(() => {
                    chrome.history.deleteUrl({ url });
                }, 500);
            }
        });
    });
}

function listAllSitesOnPage() {
    chrome.storage.sync.get(['filterSites'], function (result) {
        let currentSites = result["filterSites"];
        console.log('自定义网址有：' + currentSites);

        if (currentSites == undefined) {
            document.getElementById("fav_num").innerHTML = 0;
            var divNode = document.createElement("p");
            divNode.innerHTML = "当前无自定义网址！";
            document.getElementById("my_fav_list").appendChild(divNode);
            return;
        }

        document.getElementById("my_fav_list").innerHTML = "";

        console.log("自定义网址条数：" + currentSites.length);
        document.getElementById("fav_num").innerHTML = currentSites.length;
        for (const site of currentSites) {
            createSiteList(site);
        }
    });
}

function saveFailed(errorMsg, msg) {
    errorMsg.innerHTML = msg;
    errorMsg.style.display = "block";
    errorMsg.style.color = "#ffffff";
    setTimeout(() => {
        errorMsg.style.color = "lightcoral";
    }, 20);
}

function saveSuccessful(errorMsg, filterSites) {
    console.log('保存网址到本地成功！自定义网址有：' + filterSites);
    errorMsg.innerHTML = "添加成功！";
    errorMsg.style.display = "block";
    errorMsg.style.color = "#ffffff";
    setTimeout(() => {
        errorMsg.style.color = "#4CAF50";
    }, 20);
}

function deleteSite(site) {
    chrome.storage.sync.get(['filterSites'], function (result) {
        let currentSites = result["filterSites"];

        if (currentSites == undefined) {
            return;
        }

        currentSites.splice(currentSites.indexOf(site), 1);

        chrome.storage.sync.set({
            "filterSites": currentSites
        }, function () {
            listAllSitesOnPage();
        });
    });

}

function createSiteList(site) {
    var liNode = document.createElement("li");
    liNode.setAttribute("class", "my_fav_list_li");

    var aNode = document.createElement("a");
    aNode.setAttribute("id", "my_fav_list_a");
    aNode.setAttribute("class", "my_fav_list_a");
    aNode.setAttribute("href", "#");
    aNode.setAttribute("target", "_blank");
    aNode.innerHTML = site;

    var labelNode = document.createElement("label");
    labelNode.setAttribute("class", "my_fav_list_label");

    var buttonNode = document.createElement("button");
    buttonNode.setAttribute("class", "button");
    buttonNode.setAttribute("id", "deleteSite");
    buttonNode.addEventListener("click", function () { deleteSite(site) });
    buttonNode.innerHTML = "删除";

    labelNode.appendChild(buttonNode);
    liNode.appendChild(aNode);
    liNode.appendChild(labelNode);

    document.getElementById("my_fav_list").appendChild(liNode);
}

window.onload = function () {
    listAllSitesOnPage();
}